@@ -1,9 +1,9 @@
-/* $Id: prompt.c 4475 2010-01-17 05:30:22Z astyanax $ */
+/* $Id: prompt.c 4878 2014-05-13 21:11:59Z bens $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
  *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009 Free Software Foundation, Inc.                            *
+ *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 3, or (at your option)  *
@@ -88,7 +88,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 #endif
 
     /* Check for a shortcut in the current list. */
-    s = get_shortcut(currmenu, &input, meta_key, func_key);
+    s = get_shortcut(currmenu, &input, meta_key);
 
     /* If we got a shortcut from the current list, or a "universal"
      * statusbar prompt shortcut, set have_shortcut to TRUE. */
@@ -126,7 +126,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 	/* If we got a shortcut, or if there aren't any other characters
 	 * waiting after the one we read in, we need to display all the
 	 * characters in the input buffer if it isn't empty. */
-	 if (*have_shortcut || get_key_buffer_len() == 0) {
+	if (*have_shortcut || get_key_buffer_len() == 0) {
 	    if (kbinput != NULL) {
 		/* Display all the characters in the input buffer at
 		 * once, filtering out control characters. */
@@ -152,41 +152,41 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 	}
 
 	if (*have_shortcut) {
-	    if (s->scfunc == DO_TAB || s->scfunc == DO_ENTER)
+	    if (s->scfunc == do_tab || s->scfunc == do_enter_void)
 		;
-	    else if (s->scfunc == TOTAL_REFRESH)
+	    else if (s->scfunc == total_refresh)
 		total_statusbar_refresh(refresh_func);
-            else if (s->scfunc ==  DO_CUT_TEXT) {
+            else if (s->scfunc == do_cut_text_void) {
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Cut. */
 		if (!ISSET(RESTRICTED) || openfile->filename[0] ==
 			'\0' || currmenu != MWRITEFILE)
 		    do_statusbar_cut_text();
-	    } else if (s->scfunc == DO_RIGHT)
+	    } else if (s->scfunc == do_right)
 		do_statusbar_right();
-	    else if (s->scfunc == DO_LEFT)
+	    else if (s->scfunc == do_left)
 		do_statusbar_left();
 
 #ifndef NANO_TINY
-	    else if (s->scfunc ==  DO_NEXT_WORD)
+	    else if (s->scfunc == do_next_word_void)
 		do_statusbar_next_word(FALSE);
-	    else if (s->scfunc ==  DO_PREV_WORD)
+	    else if (s->scfunc == do_prev_word_void)
 		    do_statusbar_prev_word(FALSE);
 #endif
-	    else if (s->scfunc == DO_HOME)
+	    else if (s->scfunc == do_home)
 		    do_statusbar_home();
-	    else if (s->scfunc == DO_END)
+	    else if (s->scfunc == do_end)
 		    do_statusbar_end();
 
 #ifndef NANO_TINY
-	    else if (s->scfunc == DO_FIND_BRACKET)
+	    else if (s->scfunc == do_find_bracket)
 		do_statusbar_find_bracket();
 #endif
-	    else if (s->scfunc == DO_VERBATIM_INPUT) {
+	    else if (s->scfunc == do_verbatim_input) {
 		    /* If we're using restricted mode, the filename
 		     * isn't blank, and we're at the "Write File"
-	   	     * prompt, disable verbatim input. */
+		     * prompt, disable verbatim input. */
 		    if (!ISSET(RESTRICTED) ||
 			openfile->filename[0] == '\0' ||
 			currmenu != MWRITEFILE) {
@@ -201,18 +201,18 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 			     * to indicate that we're done. */
 			    if (got_enter) {
 				get_input(NULL, 1);
-				input = sc_seq_or(DO_ENTER, 0);
+				input = sc_seq_or(do_enter_void, 0);
 				*finished = TRUE;
 			    }
 		    }
-	    } else if (s->scfunc == DO_DELETE) {
+	    } else if (s->scfunc == do_delete) {
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Delete. */
 		if (!ISSET(RESTRICTED) || openfile->filename[0] ==
 			'\0' || currmenu != MWRITEFILE)
 		    do_statusbar_delete();
-	    } else if (s->scfunc == DO_BACKSPACE) {
+	    } else if (s->scfunc == do_backspace) {
 		/* If we're using restricted mode, the filename
 		 * isn't blank, and we're at the "Write File"
 		 * prompt, disable Backspace. */
@@ -220,17 +220,16 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 			'\0' || currmenu != MWRITEFILE)
 		    do_statusbar_backspace();
 	    } else {
-	    /* Handle the normal statusbar prompt shortcuts, setting
-	     * ran_func to TRUE if we try to run their associated
-	     * functions and setting finished to TRUE to indicate
-	     * that we're done after running or trying to run their
-	     * associated functions. */
-
+		/* Handle the normal statusbar prompt shortcuts, setting
+		 * ran_func to TRUE if we try to run their associated
+		 * functions and setting finished to TRUE to indicate
+		 * that we're done after running or trying to run their
+		 * associated functions. */
 		f = sctofunc((sc *) s);
-		if (s->scfunc != 0 &&  s->execute == TRUE) {
-			*ran_func = TRUE;
+		if (s->scfunc != 0 && s->execute == TRUE) {
+		    *ran_func = TRUE;
 		    if (f && (!ISSET(VIEW_MODE) || (f->viewok)))
-		        iso_me_harder_funcmap(f->scfunc);
+		        f->scfunc();
 		}
 		*finished = TRUE;
 	    }
@@ -320,7 +319,7 @@ void do_statusbar_output(char *output, size_t output_len, bool
 		char_buf_len)))
 	    continue;
 
-	/* More dangerousness fun =) */
+	/* More dangerousness fun. =) */
 	answer = charealloc(answer, answer_len + (char_buf_len * 2));
 
 	assert(statusbar_x <= answer_len);
@@ -854,7 +853,9 @@ void update_statusbar_line(const char *curranswer, size_t index)
     index = strnlenpt(curranswer, index);
     page_start = get_statusbar_page_start(start_col, start_col + index);
 
-    wattron(bottomwin, reverse_attr);
+    if (interface_color_pair[TITLE_BAR].bright)
+	wattron(bottomwin, A_BOLD);
+    wattron(bottomwin, interface_color_pair[TITLE_BAR].pairnum);
 
     blank_statusbar();
 
@@ -867,7 +868,8 @@ void update_statusbar_line(const char *curranswer, size_t index)
     waddstr(bottomwin, expanded);
     free(expanded);
 
-    wattroff(bottomwin, reverse_attr);
+    wattroff(bottomwin, A_BOLD);
+    wattroff(bottomwin, interface_color_pair[TITLE_BAR].pairnum);
     statusbar_pww = statusbar_xplustabs();
     reset_statusbar_cursor();
     wnoutrefresh(bottomwin);
@@ -976,115 +978,101 @@ fprintf(stderr, "get_prompt_string: answer = \"%s\", statusbar_x = %lu\n", answe
 	    &ran_func, &finished, TRUE, refresh_func);
 	assert(statusbar_x <= strlen(answer));
 
-	s = get_shortcut(currmenu, &kbinput, meta_key, func_key);
+	s = get_shortcut(currmenu, &kbinput, meta_key);
 
 	if (s)
-	    if (s->scfunc ==  CANCEL_MSG || s->scfunc == DO_ENTER)
+	    if (s->scfunc == do_cancel || s->scfunc == do_enter_void)
 		break;
 
 #ifndef DISABLE_TABCOMP
-	if (s && s->scfunc != DO_TAB)
+	if (s && s->scfunc != do_tab)
 	    tabbed = FALSE;
-#endif
 
-#ifndef DISABLE_TABCOMP
+	if (s && s->scfunc == do_tab) {
 #ifndef NANO_TINY
-	if (s && s->scfunc == DO_TAB) {
-		if (history_list != NULL) {
-		    if (last_kbinput != sc_seq_or(DO_TAB, NANO_CONTROL_I))
-			complete_len = strlen(answer);
+	    if (history_list != NULL) {
+		if (last_kbinput != sc_seq_or(do_tab, NANO_CONTROL_I))
+		    complete_len = strlen(answer);
 
-		    if (complete_len > 0) {
-			answer = mallocstrcpy(answer,
+		if (complete_len > 0) {
+		    answer = mallocstrcpy(answer,
 				get_history_completion(history_list,
-				answer, complete_len));
-			statusbar_x = strlen(answer);
-		    }
-		} else
+					answer, complete_len));
+		    statusbar_x = strlen(answer);
+		}
+	    } else
 #endif /* !NANO_TINY */
-		if (allow_tabs)
-		    answer = input_tab(answer, allow_files,
-			&statusbar_x, &tabbed, refresh_func, list);
+	    if (allow_tabs)
+		answer = input_tab(answer, allow_files, &statusbar_x,
+				   &tabbed, refresh_func, list);
 
-		update_statusbar_line(answer, statusbar_x);
-	} else 
+	    update_statusbar_line(answer, statusbar_x);
+	} else
 #endif /* !DISABLE_TABCOMP */
 #ifndef NANO_TINY
-	if (s && s->scfunc ==  PREV_HISTORY_MSG) {
-		if (history_list != NULL) {
-		    /* If we're scrolling up at the bottom of the
-		     * history list and answer isn't blank, save answer
-		     * in magichistory. */
-		    if ((*history_list)->next == NULL &&
-			answer[0] != '\0')
-			magichistory = mallocstrcpy(magichistory,
-				answer);
+	if (s && s->scfunc == get_history_older_void) {
+	    if (history_list != NULL) {
+		/* If we're scrolling up at the bottom of the history list
+		 * and answer isn't blank, save answer in magichistory. */
+		if ((*history_list)->next == NULL && answer[0] != '\0')
+		    magichistory = mallocstrcpy(magichistory, answer);
 
-		    /* Get the older search from the history list and
-		     * save it in answer.  If there is no older search,
-		     * don't do anything. */
-		    if ((history =
-			get_history_older(history_list)) != NULL) {
-			answer = mallocstrcpy(answer, history);
-			statusbar_x = strlen(answer);
-		    }
-
-		    update_statusbar_line(answer, statusbar_x);
-
-		    /* This key has a shortcut list entry when it's used
-		     * to move to an older search, which means that
-		     * finished has been set to TRUE.  Set it back to
-		     * FALSE here, so that we aren't kicked out of the
-		     * statusbar prompt. */
-		    finished = FALSE;
+		/* Get the older search from the history list and save it in
+		 * answer.  If there is no older search, don't do anything. */
+		if ((history = get_history_older(history_list)) != NULL) {
+		    answer = mallocstrcpy(answer, history);
+		    statusbar_x = strlen(answer);
 		}
-	} else if (s && s->scfunc ==  NEXT_HISTORY_MSG) {
-		if (history_list != NULL) {
-		    /* Get the newer search from the history list and
-		     * save it in answer.  If there is no newer search,
-		     * don't do anything. */
-		    if ((history =
-			get_history_newer(history_list)) != NULL) {
-			answer = mallocstrcpy(answer, history);
-			statusbar_x = strlen(answer);
-		    }
 
-		    /* If, after scrolling down, we're at the bottom of
-		     * the history list, answer is blank, and
-		     * magichistory is set, save magichistory in
-		     * answer. */
-		    if ((*history_list)->next == NULL &&
-			*answer == '\0' && magichistory != NULL) {
+		update_statusbar_line(answer, statusbar_x);
+
+		/* This key has a shortcut-list entry when it's used to
+		 * move to an older search, which means that finished has
+		 * been set to TRUE.  Set it back to FALSE here, so that
+		 * we aren't kicked out of the statusbar prompt. */
+		 finished = FALSE;
+	    }
+	} else if (s && s->scfunc == get_history_newer_void) {
+	    if (history_list != NULL) {
+		/* Get the newer search from the history list and save it in
+		 * answer.  If there is no newer search, don't do anything. */
+		if ((history = get_history_newer(history_list)) != NULL) {
+		    answer = mallocstrcpy(answer, history);
+		    statusbar_x = strlen(answer);
+		}
+
+		/* If, after scrolling down, we're at the bottom of the
+		 * history list, answer is blank, and magichistory is set,
+		 * save magichistory in answer. */
+		if ((*history_list)->next == NULL &&
+		    *answer == '\0' && magichistory != NULL) {
 			answer = mallocstrcpy(answer, magichistory);
 			statusbar_x = strlen(answer);
 		    }
 
-		    update_statusbar_line(answer, statusbar_x);
-
-		    /* This key has a shortcut list entry when it's used
-		     * to move to a newer search, which means that
-		     * finished has been set to TRUE.  Set it back to
-		     * FALSE here, so that we aren't kicked out of the
-		     * statusbar prompt. */
-		    finished = FALSE;
-		}
-	} else
-#endif /* !NANO_TINY */
-	if (s && s->scfunc == DO_HELP_VOID) {
 		update_statusbar_line(answer, statusbar_x);
 
-		/* This key has a shortcut list entry when it's used to
-		 * go to the help browser or display a message
-		 * indicating that help is disabled, which means that
-		 * finished has been set to TRUE.  Set it back to FALSE
-		 * here, so that we aren't kicked out of the statusbar
-		 * prompt. */
+		/* This key has a shortcut-list entry when it's used to
+		 * move to a newer search, which means that finished has
+		 * been set to TRUE.  Set it back to FALSE here, so that
+		 * we aren't kicked out of the statusbar prompt. */
 		finished = FALSE;
+	    }
+	} else
+#endif /* !NANO_TINY */
+	if (s && s->scfunc == do_help_void) {
+	    update_statusbar_line(answer, statusbar_x);
+
+	    /* This key has a shortcut-list entry when it's used to go to
+	     * the help browser or display a message indicating that help
+	     * is disabled, which means that finished has been set to TRUE.
+	     * Set it back to FALSE here, so that we aren't kicked out of
+	     * the statusbar prompt. */
+	    finished = FALSE;
 	}
 
-	/* If we have a shortcut with an associated function, break out
-	 * if we're finished after running or trying to run the
-	 * function. */
+	/* If we have a shortcut with an associated function, break out if
+	 * we're finished after running or trying to run the function. */
 	if (finished)
 	    break;
 
@@ -1098,8 +1086,8 @@ fprintf(stderr, "get_prompt_string: answer = \"%s\", statusbar_x = %lu\n", answe
 
 
 #ifndef NANO_TINY
-    /* Set the current position in the history list to the bottom and
-     * free magichistory, if we need to. */
+    /* Set the current position in the history list to the bottom,
+     * and free magichistory if we need to. */
     if (history_list != NULL) {
 	history_reset(*history_list);
 
@@ -1114,7 +1102,7 @@ fprintf(stderr, "get_prompt_string: answer = \"%s\", statusbar_x = %lu\n", answe
      * we've finished putting in an answer, reset the statusbar cursor
      * position too. */
     if (s) {
-	if (s->scfunc ==  CANCEL_MSG || s->scfunc == DO_ENTER ||
+	if (s->scfunc == do_cancel || s->scfunc == do_enter_void ||
 	ran_func) {
 	    statusbar_x = old_statusbar_x;
 	    statusbar_pww = old_pww;
@@ -1201,9 +1189,9 @@ int do_prompt(bool allow_tabs,
 
     /* If we left the prompt via Cancel or Enter, set the return value
      * properly. */
-    if (s && s->scfunc ==  CANCEL_MSG)
+    if (s && s->scfunc == do_cancel)
 	retval = -1;
-    else if (s && s->scfunc == DO_ENTER)
+    else if (s && s->scfunc == do_enter_void)
 	retval = (*answer == '\0') ? -2 : 0;
 
     blank_statusbar();
@@ -1288,12 +1276,15 @@ int do_yesno_prompt(bool all, const char *msg)
 	onekey("^C", _("Cancel"), width);
     }
 
-    wattron(bottomwin, reverse_attr);
+    if (interface_color_pair[TITLE_BAR].bright)
+	wattron(bottomwin, A_BOLD);
+    wattron(bottomwin, interface_color_pair[TITLE_BAR].pairnum);
 
     blank_statusbar();
     mvwaddnstr(bottomwin, 0, 0, msg, actual_x(msg, COLS - 1));
 
-     wattroff(bottomwin, reverse_attr);
+    wattroff(bottomwin, A_BOLD);
+    wattroff(bottomwin, interface_color_pair[TITLE_BAR].pairnum);
 
     /* Refresh the edit window and the statusbar before getting
      * input. */
@@ -1309,9 +1300,9 @@ int do_yesno_prompt(bool all, const char *msg)
 
 	currmenu = MYESNO;
 	kbinput = get_kbinput(bottomwin, &meta_key, &func_key);
-	s = get_shortcut(currmenu, &kbinput, &meta_key, &func_key);
+	s = get_shortcut(currmenu, &kbinput, &meta_key);
 
-	if (s && s->scfunc ==  CANCEL_MSG)
+	if (s && s->scfunc == do_cancel)
 	    ok = -1;
 #ifndef DISABLE_MOUSE
 	else if (kbinput == KEY_MOUSE) {
@@ -1343,7 +1334,7 @@ int do_yesno_prompt(bool all, const char *msg)
 		}
 	}
 #endif /* !DISABLE_MOUSE */
-	else if  (s && s->scfunc == TOTAL_REFRESH) {
+	else if  (s && s->scfunc == total_refresh) {
 	    total_redraw();
 	    continue;
 	} else {
